/*
  pqrdipole - pqrdipole.c
  Copyright (c) 2010 Arkadiusz Bokowy

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  If you want to read full version of the GNU General Public License
  see <http://www.gnu.org/licenses/>.

  ** Note: **
  For contact information and the latest version of this program see
  my webpage <http://arkq.awardspace.us/pdblib>.

  ** Synopsis: **
  This program calculate the overall dipole of the molecule in PQR file.
  Reference point is geometrical center of the molecule or user defined point.

  usage: pqrdipole PQR_file [x y z]
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "pdblib.h"

int atom_counter = 0;
static int calculate_GC_callback(PDBhandle PDBh, PDBlib_iter_data_t *data)
{
	double *refer_point = (double*)data->userdata;
	int i;

	for(i = 0; i < 3; i++) refer_point[i] += data->a->coordXYZ[i];
	atom_counter++;
	return 0;
}

static int sum_charge_callback(PDBhandle PDBh, PDBlib_iter_data_t *data)
{
	*((double*)data->userdata) += data->a->occupancy;
	return 0;
}

struct dipole_calc_data {
	double refer_point[3];
	double dipole_xyz[3], dipole_len, dipole_len_debye;
};
static int calculate_dipole_callback(PDBhandle PDBh, PDBlib_iter_data_t *data)
{
	struct dipole_calc_data *calc = (struct dipole_calc_data*)data->userdata;
	int i;

	// elementary_charge = 1.6021764874E-19
	// elementary_charge*1E-10 = 1.6021764874E-29
	for(i = 0; i < 3; i++)
		calc->dipole_xyz[i] += (data->a->coordXYZ[i] - calc->refer_point[i])*
				data->a->occupancy*1.6021764874E-29;
	return 0;
}

int main(int argc, char *argv[])
{
	PDBhandle PQRh;
	struct dipole_calc_data calc;
	double molecule_charge;
	int i;

	bzero(&calc, sizeof(calc));

	// parse command line arguments
	if(argc == 5)
		for(i = 0; i < 3; i++) calc.refer_point[i] = atof(argv[2+i]);

	if(argc != 2 && argc != 5){
		printf("usage: %s PQR_file [x y z]\n", argv[0]);
		return 0;}

	PQRh = PDBlib_initialize();

	if(PDBlib_load_file(PQRh, argv[1], 1) == -1){
		printf("error: unable to open PQR file `%s`\n", argv[1]);
		PDBlib_free(PQRh);
		return 1;}

	// calculate geometrical center of molecule
	if(argc == 2){
		PDBlib_iterator(PQRh, PDBLIB_ITER_ATOM, calculate_GC_callback, calc.refer_point);
		for(i = 0; i < 3; i++) calc.refer_point[i] /= atom_counter;}

	// calculate overall charge on molecule
	molecule_charge = 0;
	PDBlib_iterator(PQRh, PDBLIB_ITER_ATOM, sum_charge_callback, &molecule_charge);

	// calculate overall dipole moment of molecule
	PDBlib_iterator(PQRh, PDBLIB_ITER_ATOM, calculate_dipole_callback, &calc);
	calc.dipole_len = sqrt(calc.dipole_xyz[0]*calc.dipole_xyz[0] +
			calc.dipole_xyz[1]*calc.dipole_xyz[1] +
			calc.dipole_xyz[2]*calc.dipole_xyz[2]);
	calc.dipole_len_debye = calc.dipole_len*2.99792458E29;
	for(i = 0; i < 3; i++) calc.dipole_xyz[i] /= calc.dipole_len;

	printf("Total charge on molecule [e]: % f\n", molecule_charge);
	printf("Coordinations of referencing point [A]: [% f % f % f] (%s)\n",
		calc.refer_point[0], calc.refer_point[1], calc.refer_point[2],
		argc == 2 ? "GC" : "user defined");
	printf("Components of dipole moment [normalized]: [% .8f % .8f % .8f]\n",
			calc.dipole_xyz[0], calc.dipole_xyz[1], calc.dipole_xyz[2]);
	printf("Total dipole moment [Cm]: %.6e (%f D)\n",
			calc.dipole_len, calc.dipole_len_debye);

	PDBlib_free(PQRh);
	return 0;
}
